% clean up the matlab environment
clear; clc; close all;

% run initialization of some paths and variables
init_setup;

H = 400;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% we can start working now %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% let's use DDP to find a trajectory controller: we 'll fly a straight line, starting and ending in hover %% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Q3: 
%% Q3a: hand in a plot of the target and achieved trajectories: ned, q, and
%% u_prev; 
%% Q3b: and a printout of the code (this m-file)
%% Q3c: Comment briefly on how well they match---if in some axes they don't
%% match, explain why this might be the case.
%% 


%% fill in target_traj.x, target_traj.u, target_traj.t
%%
%% (1) we hover for 5 seconds (100 tics) at (0,0,0)
%% (2) then fly to point (10,10,10); start time: 5s, end time: 10s [100 tics]
%% (3) then we change heading: from facing north (q = [0;0;0;1]) to facing
%% East; start time: 10s; end time: 15s; [100 tics]
%% (4) then we hover at (10,10,10) while facing East; start time: 20s, end
%% time: 20s;
%% [--> total number of tics is 600]

%% fill everything in with the default hover:
hover_quaternion_target = [0;0;0;1]; 
target_hover_state = [ zeros(4,1); zeros(4,1); zeros(3,1); zeros(3,1); zeros(3,1); hover_quaternion_target];
target_traj.x = repmat(target_hover_state',H,1);
target_traj.u = zeros(H,4); %% recall: u is change in controls, which is zero for stationary hover
target_traj.t = ((0:length(target_traj.x)-1)*dt)';

% overwrite some target state variables for the linear move:
t_move_start = 101;
t_move = 100;
target_traj.x(t_move_start:t_move_start+t_move-1,idx.ned(1)) = 0:10/(t_move-1):10;
target_traj.x(t_move_start:t_move_start+t_move-1,idx.ned(2)) = 
target_traj.x(t_move_start:t_move_start+t_move-1,idx.ned(3)) = 
target_traj.x(t_move_start:t_move_start+t_move-1,idx.ned_dot) =

% overwrite some target state variables for the in-place rotation:
t_rot_start = 201;
t_rot = 100;
target_traj.x(t_rot_start:t_rot_start+t_rot-1,idx.ned) = 10;
target_traj.x(t_rot_start:t_rot_start+t_rot-1,idx.pqr(3)) = 
rot_phase = 0:(pi/2)/(t_rot-1):pi/2;
target_traj.x(t_rot_start:t_rot_start+t_rot-1,idx.q(3)) = 
target_traj.x(t_rot_start:t_rot_start+t_rot-1,idx.q(4)) = 

% overwrite some target state variables for the hover at new pos and
% orientation:
t_fhover_start = 301;
t_fhover = 100;
target_traj.x(t_fhover_start:t_fhover_start+t_fhover-1,idx.ned) = 
target_traj.x(t_fhover_start:t_fhover_start+t_fhover-1,idx.q(3)) =
target_traj.x(t_fhover_start:t_fhover_start+t_fhover-1,idx.q(4)) =



%% let's run DDP:

start_state = target_hover_state;
model_bias_est = zeros(H,6);

u_prev_mult = 0; u_delta_prev_mult = 1000;
ned_dot_mult = 1; ned_mult = 1;
pqr_mult = 1; q_mult = 1;
always1state_mult = 0;

reward.state_multipliers = [ u_prev_mult * ones(1,4)   u_delta_prev_mult * ones(1,4)  ned_dot_mult * ones(1,3)  ned_mult*ones(1,3)  ...
	pqr_mult * ones(1,3)  q_mult * ones(1,3)  always1state_mult * ones(1,1)]';
reward.input_multipliers = ones(4,1)*0;

simulate_f = @f_heli;

[lqr_traj, result_traj, im_trajs] = ddp_for_trajectory_following(start_state, reward, target_traj, simulate_f, model, idx, model_bias_est, dt);

% let's check how well we perform

x = start_state; 
for i=1:H-1
  dx = compute_dx(lqr_traj.target_x(i,:)', x);
  u = (lqr_traj.K{i}*dx);
  traj.x(i,:) = x';
  x = simulate_f(x, u, dt, model, idx);
end
traj.x(H,:) = x';


figure; plot(traj.x(:,idx.ned)); legend('north', 'east', 'down');
hold on; plot(target_traj.x(:,idx.ned),'.');

figure; plot(traj.x(:,idx.q)); legend('qx', 'qy', 'qz', 'qw');
hold on; plot(target_traj.x(:,idx.q),'.');

figure; plot(traj.x(:,idx.u_prev)); legend('aileron','elevator','rudder','collective');
hold on; plot(target_traj.x(:,idx.u_prev),'.');








